@@ -7,15 +7,14 @@ from copy import deepcopy
 from typing import Optional
 from cid import is_cid
 
-from transactions.common.transaction import Transaction
+from transactions.common.transaction import Transaction, COMPOSE
 from transactions.common.input import Input
 from transactions.common.output import Output
 from transactions.common.schema import validate_transaction_schema
-from transactions.common.exceptions import SchemaValidationError
 
 
 class Compose(Transaction):
-    OPERATION = "COMPOSE"
+    OPERATION = COMPOSE
     ALLOWED_OPERATIONS = (OPERATION,)
 
     @classmethod
@@ -61,6 +60,7 @@ class Compose(Transaction):
         recipients: list[tuple[list[str], int]],
         assets: list[str],
         metadata: Optional[dict] = None,
+        script: Optional[dict] = None,
     ):
         asset_ids = []
         new_assets = []
@@ -72,6 +72,6 @@ class Compose(Transaction):
         (inputs, outputs) = Compose.validate_compose(inputs, recipients, new_assets, asset_ids)
         new_assets = [{"data": cid} for cid in new_assets]
         asset_ids = [{"id": id} for id in asset_ids]
-        compose = cls(cls.OPERATION, new_assets + asset_ids, inputs, outputs, metadata)
+        compose = cls(cls.OPERATION, new_assets + asset_ids, inputs, outputs, metadata, script=script)
         cls.validate_schema(compose.to_dict())
         return compose
