@@ -7,15 +7,14 @@ from copy import deepcopy
 from typing import Optional
 from cid import is_cid
 
-from transactions.common.transaction import Transaction
+from transactions.common.transaction import Transaction, DECOMPOSE
 from transactions.common.input import Input
 from transactions.common.output import Output
 from transactions.common.schema import validate_transaction_schema
-from transactions.common.exceptions import SchemaValidationError
 
 
 class Decompose(Transaction):
-    OPERATION = "DECOMPOSE"
+    OPERATION = DECOMPOSE
     ALLOWED_OPERATIONS = (OPERATION,)
 
     @classmethod
@@ -70,6 +69,7 @@ class Decompose(Transaction):
         recipients: list[tuple[list[str], int]],
         assets: list[str],
         metadata: Optional[dict] = None,
+        script: Optional[dict] = None,
     ):
         asset_ids = []
         new_assets = []
@@ -81,6 +81,6 @@ class Decompose(Transaction):
         (inputs, outputs) = Decompose.validate_decompose(inputs, recipients, new_assets, asset_ids)
         new_assets = [{"data": cid} for cid in new_assets]
         asset_ids = [{"id": id} for id in asset_ids]
-        decompose = cls(cls.OPERATION, new_assets + asset_ids, inputs, outputs, metadata)
+        decompose = cls(cls.OPERATION, new_assets + asset_ids, inputs, outputs, metadata, script=script)
         cls.validate_schema(decompose.to_dict())
         return decompose
