@@ -5,14 +5,14 @@
 
 from typing import Optional
 
-from transactions.common.transaction import Transaction
+from transactions.common.transaction import Transaction, TRANSFER
 from transactions.common.input import Input
 from transactions.common.output import Output
 from copy import deepcopy
 
 
 class Transfer(Transaction):
-    OPERATION = "TRANSFER"
+    OPERATION = TRANSFER
     ALLOWED_OPERATIONS = (OPERATION,)
 
     @classmethod
@@ -53,6 +53,7 @@ class Transfer(Transaction):
         recipients: list[tuple[list[str], int]],
         asset_ids: list[str],
         metadata: Optional[dict] = None,
+        script: Optional[dict] = None,
     ):
         """A simple way to generate a `TRANSFER` transaction.
 
@@ -93,4 +94,4 @@ class Transfer(Transaction):
         """
         (inputs, outputs) = cls.validate_transfer(inputs, recipients, asset_ids, metadata)
         ids = [{"id": id} for id in asset_ids]
-        return cls(cls.OPERATION, ids, inputs, outputs, metadata)
+        return cls(cls.OPERATION, ids, inputs, outputs, metadata, script=script)
